/**********************************************************/
/*                        T I A R E                       */
/* Astronomical Ressources & Ephemeris for TI Calculators */
/*               (c) 2003-2005, Yann DUCHEMIN             */
/* ------------------------------------------------------ */
/* This file is part of TIARE.                            */
/*                                                        */
/* TIARE is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public    */
/* License as published by the Free Software Foundation;  */
/* either version 2 of the License                        */
/*                                                        */
/* TIARE is distributed in the hope that it will be       */
/* useful, but WITHOUT ANY WARRANTY; without even the     */
/* implied warranty of MERCHANTABILITY or FITNESS FOR A   */
/* PARTICULAR PURPOSE. See the GNU General Public         */
/* License for more details.                              */
/*                                                        */
/* You should have received a copy of the GNU General     */
/* Public License along with TIARE; if not, write to      */
/* the Free Software Foundation, Inc., 59 Temple Place,   */
/* Suite 330, Boston, MA  02111-1307  USA                 */
/* ------------------------------------------------------ */
/* File Name : errors.c                                   */
/* Created   : 01 III  2003                               */
/* Modified  : 10  IV  2005                               */
/**********************************************************/

#include "errors.h"

/* ====================================================== */
/* = Function    : Error Box Message                    = */
/* = Input       : Ref Value defined in errors.h        = */
/* = Output      : -                                    = */
/* ====================================================== */
void ErrMsg(int type, int value)
{
 DlgMessage ((char *)msg_err[(int)type],(char *)msg_err[(int)value], BT_NONE, BT_OK);
}

/* ====================================================== */
/* = Function    : Time Validation                      = */
/* = Input       : time struct                          = */
/* = Output      : state                                = */
/* ====================================================== */
short VerifyTimeStruct(struct TIME *time)
{
 // control values entered
 if( !((short)time->ObsSec  >= 0 && (short)time->ObsSec  <= 59) )
 { return(MSG_SEC); }
 if( !((short)time->ObsMin  >= 0 && (short)time->ObsMin  <= 59) )
 { return(MSG_MIN); }
 if( !((short)time->ObsHour >= 0 && (short)time->ObsHour <= 23) )
 { return(MSG_HRS); }
 if( !((short)time->ObsYear >= 1900) ) // it's better to keep a good precision
 { return(MSG_YER); }
 if( !((short)time->ObsMonth > 0 && (short)time->ObsMonth <=12) )
 { return(MSG_MTH); }
 if( !((short)time->ObsDay > 0 && ((short)time->ObsDay <= NbDaysFMonth(time->ObsMonth, time->ObsYear)) ) )
 { return(MSG_DAY); }

 return(0);
}

/* ====================================================== */
/* = Function    : Local Validation                     = */
/* = Input       : place struct                         = */
/* = Output      : state                                = */
/* ====================================================== */
short VerifyPlaceStruct(struct PLACE *pplace)
{
	if( !((short)pplace->LatDeg >= -90 && (short)pplace->LatDeg <= 90))
  {   return(MSG_KEY); }
	if( !((short)pplace->LongDeg >= -180  && (short)pplace->LongDeg <= 180) )
  {   return(MSG_KEY); }
  if( !((short)pplace->LatMin >= 0  && (short)pplace->LatMin <= 59) )
  {   return(MSG_KEY); }
	if( !((short)pplace->LatSec >= 0 && (short)pplace->LatSec <= 59) )
	{   return(MSG_KEY); }
  if( !((short)pplace->LongMin >= 0  && (short)pplace->LongMin <= 59) )
  {   return(MSG_KEY); }
	if( !((short)pplace->LongSec >= 0 && (short)pplace->LongSec <= 59) )
  {		return(MSG_KEY); }
  if( !((short)pplace->DTime >= -12 && (short)pplace->DTime <= 12 ) )
 	{		return(MSG_KEY); }
 return(0);
}