/**********************************************************/
/*                        T I A R E                       */
/* Astronomical Ressources & Ephemeris for TI Calculators */
/*               (c) 2003-2005, Yann DUCHEMIN             */
/* ------------------------------------------------------ */
/* This file is part of TIARE.                            */
/*                                                        */
/* TIARE is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public    */
/* License as published by the Free Software Foundation;  */
/* either version 2 of the License                        */
/*                                                        */
/* TIARE is distributed in the hope that it will be       */
/* useful, but WITHOUT ANY WARRANTY; without even the     */
/* implied warranty of MERCHANTABILITY or FITNESS FOR A   */
/* PARTICULAR PURPOSE. See the GNU General Public         */
/* License for more details.                              */
/*                                                        */
/* You should have received a copy of the GNU General     */
/* Public License along with TIARE; if not, write to      */
/* the Free Software Foundation, Inc., 59 Temple Place,   */
/* Suite 330, Boston, MA  02111-1307  USA                 */
/* ------------------------------------------------------ */
/* File Name : fich.c                                     */
/* Created   : 27 II  2003                                */
/* Modified  : 10 IV  2005                                */
/**********************************************************/

#include "fich.h"

/* ====================================================== */
/* = Function    : Open FichTime                        = */
/* = Input       : TIME struct                          = */
/* = Output      : state                                = */
/* ====================================================== */
short RFchTime(struct TIME *time)
{
 FILE *fchtime = NULL;
 // open file for reading in binary mode
 if ((fchtime = fopen("ficht","rb")) == NULL)	return(1);
 else
 // read data from file into TIME structure
 if (fread(time,sizeof(struct TIME),1,fchtime) != 1) return(1);
 fclose(fchtime);
 return(0);
}

/* ====================================================== */
/* = Function    : Open FichLocal                       = */
/* = Input       : PLACE struct                         = */
/* = Output      : state                                = */
/* ====================================================== */
short RFchLocal(struct PLACE *pplace)
{
 FILE *fchlocal = NULL;
 // open file for reading in binary mode
 if ((fchlocal = fopen("fichl","rb")) == NULL)	return(1);
 else
 // read data from file into TIME structure
 if (fread(pplace,sizeof(struct PLACE),1,fchlocal) != 1) return(1);
 fclose(fchlocal);
 return(0);
}

/* ====================================================== */
/* = Function    : Open FichExtrasParameters            = */
/* = Input       : PARAM struct                         = */
/* = Output      : state                                = */
/* ====================================================== */
short RFchParam(struct PARAM *param)
{
 FILE *fchparam = NULL;
 // open file for reading in binary mode
 if ((fchparam = fopen("fichp","rb")) == NULL)	return(1);
 else
 // read data from file into PARAM structure
 if (fread(param,sizeof(struct PARAM),1,fchparam) != 1) return(1);
 fclose(fchparam);
 return(0);
}

/* ====================================================== */
/* = Function    : Create or Erase Fich Time            = */
/* = Input       : time struct, time zone               = */
/* = Output      : fichT                                = */
/* ====================================================== */
short FchTime(struct TIME *time, short TZ)
{
 struct TIME Tutc;
 struct TIME *ptutc;
 ptutc = &Tutc;
 FILE *fchtime = NULL;
  
 // convert to UT
 time->UTDay   = time->ObsDay;
 time->UTMonth = time->ObsMonth;
 time->UTYear  = time->ObsYear;
 time->UTHour  = time->ObsHour;
 time->UTMin   = time->ObsMin;
 time->UTSec   = time->ObsSec;
 UTC(time, TZ);
 
 // open file for writing
 if ((fchtime = fopen("ficht","wb")) == NULL)
	return(MSG_FILE);
 else
 {
  // write structure data to the file
  if (fwrite(time,sizeof(struct TIME),1,fchtime) != 1)
	 return(MSG_FILE);
  // append the file ID tag
  fputc(0,fchtime);
  fputc(INT_TAG,fchtime);
  // close the file
  fclose(fchtime);
 }
 return(0);
}

/* ====================================================== */
/* = Function    : Create or Erase Obs Localisation     = */
/* = Input       : local struct                         = */
/* = Output      : fichL                                = */
/* ====================================================== */
short FchLocal(struct PLACE *pplace)
{
 FILE *fchlocal = NULL;
 // open file for writing
 if ((fchlocal = fopen("fichl","wb")) == NULL)
	return(MSG_FILE);
 else
 {
  // write structure data to the file
  if (fwrite(pplace,sizeof(struct PLACE),1,fchlocal) != 1)
	 return(MSG_FILE);
  // append the file ID tag
  fputc(0,fchlocal);
  fputc(INT_TAG,fchlocal);
  // close the file
  fclose(fchlocal);
 }
 return(0);
}

/* ====================================================== */
/* = Function    : Create or Erase Fich Param           = */
/* = Input       : PARAM struct                         = */
/* = Output      : fichP                                = */
/* ====================================================== */
short FchParam(struct PARAM *param)
{
 FILE *fchparam = NULL;
 if(param->Zoom > ZOOM_MAX) // limit of the zoom factor
  param->Zoom = 1;
 // open file for writing
 if ((fchparam = fopen("fichp","wb")) == NULL)
	return(MSG_FILE);
 else
 {
  // write structure data to the file
  if (fwrite(param,sizeof(struct PARAM),1,fchparam) != 1)
	 return(MSG_FILE);
  // append the file ID tag
  fputc(0,fchparam);
  fputc(INT_TAG,fchparam);
  // close the file
  fclose(fchparam);
 }
 return(0);
}