/**********************************************************/
/*                        T I A R E                       */
/* Astronomical Ressources & Ephemeris for TI Calculators */
/*               (c) 2003-2005, Yann DUCHEMIN             */
/* ------------------------------------------------------ */
/* This file is part of TIARE.                            */
/*                                                        */
/* TIARE is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public    */
/* License as published by the Free Software Foundation;  */
/* either version 2 of the License                        */
/*                                                        */
/* TIARE is distributed in the hope that it will be       */
/* useful, but WITHOUT ANY WARRANTY; without even the     */
/* implied warranty of MERCHANTABILITY or FITNESS FOR A   */
/* PARTICULAR PURPOSE. See the GNU General Public         */
/* License for more details.                              */
/*                                                        */
/* You should have received a copy of the GNU General     */
/* Public License along with TIARE; if not, write to      */
/* the Free Software Foundation, Inc., 59 Temple Place,   */
/* Suite 330, Boston, MA  02111-1307  USA                 */
/* ------------------------------------------------------ */
/* File Name : jupiter.c                                  */
/* Created   : 26 III 2003                                */
/* Modified  : 06 V   2005                                */
/**********************************************************/

#include "jupiter.h"

/* ====================================================== */
/* = Function    : JSat                                 = */
/* = Input       : TIME & Observer structures           = */
/* = Output      : state                                = */
/* ====================================================== */
short JSat(struct TIME *ptime, struct PLACE *pobs, struct PARAM *pparam)
{
 char *buffer = NULL; //output buffer
 char SatName[] = "   ";
 char *pSatName = SatName;

 float jd = 0.0,			//julian day
       T = 0.0,
       Dt = 0.0,
       d  = 0.0,
       v  = 0.0,
       m  = 0.0,
       n  = 0.0,
       j  = 0.0,
       a  = 0.0,
       b  = 0.0,
       k  = 0.0,
       re = 0.0,
       r  = 0.0,
       delta = 0.0,
       lum = 0.0,
       psi = 0.0,
       sp2 = 0.0,
       la = 0.0,		// meridian SYS I
       lb = 0.0,		// meridian SYS II
       grs = 0.0,
       lmb= 0.0,
       ds = 0.0,
       de = 0.0,
       u1 = 0.0, u1c = 0.0,
       u2 = 0.0, u2c = 0.0,
       u3 = 0.0, u3c = 0.0,
       u4 = 0.0, u4c = 0.0,
       g = 0.0,
       h = 0.0;
       
 float x1 = 0.0, x2 = 0.0, x3 = 0.0, x4 = 0.0,
       y1 = 0.0, y2 = 0.0, y3 = 0.0, y4 = 0.0;
       
 float r1 = 0.0,
       r2 = 0.0,
       r3 = 0.0,
       r4 = 0.0;

 short SCR_X = LCD_WIDTH,
 			 SCR_Y = LCD_HEIGHT,
 			 SCRX2 = SCR_X>>1,
 			 SCRY2 = SCR_Y>>1;
 float appd = 0.0;

 // allocate memory for the buffer
 if ((buffer = (char *)calloc(60,sizeof(char))) == NULL)
	return(MSG_MEM);

// ---
// the next part are issued from Jean Meeus' book
// relatives to Astronomical's Algorithms (S.A.F. ed.)
// all must be compute in degrees angles, so a conversion required !
// ---

// UTC(ptutc, pobs->DTime);		// to find the UTC with the parameters was entered

 jd = YMD2JD(ptime, REALTIME);	// find the corresponding julian's date

 d = jd-2415020;							// century since 31 XII 1899 at 12h00 TE
 
 T = d/36525.0;

 // Delta Time corresponding to the local sideral time corrected
 Dt =  RSTG_H(STG_H(jd), DMS2Float(pobs->LongDeg, pobs->LongMin, pobs->LongSec), ptime);
 
 v  = NormDeg(134.63+0.00111587*d);			// arg terme periodical mvt of Jupiter
 m  = NormDeg(358.476+0.9856003*d);			// abnormal term for earth
 n  = NormDeg(225.328+0.0830853*d+0.33*sinD(v));	// abnormal term for jupiter
 j  = NormDeg(221.647+0.9025179*d-0.33*sinD(v));	// Long helio moy of earth & jupiter
 
 a = (1.916*sinD(m)+0.020*sinD(2*m));	 // equation for earth center
 b = (5.552*sinD(n)+0.167*sinD(2*n));  // equation for jupiter center 
 k = (j+a-b);									 											   // correction
 
 re = (1.00014-0.01672*cosD(m)-0.00014*cosD((2*m)));		// earth's vector
 r  = (5.20867-0.25192*cosD(n)-0.00610*cosD((2*n)));		// jupiter's vector
 
 delta = sqrt(((r*r)+(re*re)-(2*r*re)*cosD(k)));			// distance earth-jupiter
 
 lum = (d-(delta/173.0)) ;		// light curve correction
 
 psi = (re/delta)*sinD(k);
 psi = NormDeg(Rad2Deg(asin((psi))));		// phase angle in degrees
 
 la = NormDeg( (268.28 + 877.8169088 * lum + psi - (b)) );	// Meridian SYS I
 lb = NormDeg( (290.28 + 870.1869147 * lum + psi - (b)) );  // Meridian SYS II

 sp2 = 57.3*((sinD((psi/2.0))*sinD((psi/2.0))));
  
 if ( k < 0.0 )
 {
 	la += sp2;
 	lb += sp2;
 }
 else
 {
 	la -= sp2;
 	lb -= sp2;
 }
 
 la = NormDeg((la+0.01016*Dt)); // phase correction & UT correction
 lb = NormDeg((lb+0.01007*Dt));

 // actual grs position
 grs = NormDeg(lb-pparam->GRS_Ref);
  
 FontSetSys(F_4x6);
 sprintf(buffer,"Sys.I: %0.02f",la);
 DrawStr(2,2,buffer, A_XOR);
 sprintf(buffer,"Sys.II: %0.02f",lb);
 DrawStr(2,10,buffer, A_XOR);

 lmb = NormDeg( (238.05 + 0.083091 * d + 0.3 * sinD(v) + b) );
 ds = (3.07*sinD(lmb+44.5));
 de = ( ds-2.15*sinD(psi)*cosD(lmb+24)-1.31*(r-delta)/delta*sinD(lmb-99.4) );
 
 u1 = NormDeg( (84.5506 + 203.4058630 * lum + psi - b) );
 u2 = NormDeg( (41.5015 + 101.2916323 * lum + psi - b) );
 u3 = NormDeg( (109.9770 + 50.2345169 * lum + psi - b) );
 u4 = NormDeg( (176.3586 + 21.4879802 * lum + psi - b) );

 g  = NormDeg( (187.3+50.310674*lum) );
 h  = NormDeg( (311.1+21.569229*lum) );
 
 // make a coordinates corrections with the originals values
 u1c = NormDeg(u1 + (0.472 * sinD(2*(u1-u2))) );
 u2c = NormDeg(u2 + (1.073 * sinD(2*(u2-u3))) );
 u3c = NormDeg(u3 + (0.174 * sinD(g)) );
 u4c = NormDeg(u4 + (0.845 * sinD(h)) );

 // distance from Jupiter's centre in circle-ray
 r1 = 5.9061-0.0244*cosD(2*(u1-u2));
 r2 = 9.3972-0.0889*cosD(2*(u2-u3));
 r3 = 14.9894-0.0227*cosD(g);
 r4 = 26.3649-0.1944*cosD(h);
 
 // rectangular apparent coordinates 
 x1 = r1*sinD(u1c);
 x2 = r2*sinD(u2c);
 x3 = r3*sinD(u3c);
 x4 = r4*sinD(u4c);
 
 y1 = (-r1*cosD(u1c)*sinD(de));
 y2 = (-r2*cosD(u2c)*sinD(de));
 y3 = (-r3*cosD(u3c)*sinD(de));
 y4 = (-r4*cosD(u4c)*sinD(de));
 
 // compute the apparent diameter from earth in arcseconds
 // 196.94" equ and 185.08" pol
 // this is the apparents D at 1UA from the geocentric position
 appd = (JDIAMSEC/delta);

 //validate the planet's size after zooming
 if(( d=(float)(JUPITERD*pparam->Zoom) ) > (float)SCRY2)
  d=JUPITERD;
  
 // Drawing Jupiter
 DrawCircle(SCRX2,  SCRY2, d);
 
 // Drawing the great red spot
 DrawGRS(grs, d);

 // Drawing Satellites
 strcpy(pSatName, "I");
 DrawJSat(x1, y1, d, u1c, A_NORMAL, pSatName);
 DrawJSatShadow(r1, d, (u1c - psi), de);
 strcpy(pSatName, "II");
 DrawJSat(x2, y2, d, u2c, A_NORMAL, pSatName);
 DrawJSatShadow(r2, d, (u2c - psi), de);
 strcpy(pSatName, "III");
 DrawJSat(x3, y3, d, u3c, A_NORMAL, pSatName);
 DrawJSatShadow(r3, d, (u3c - psi), de);
 strcpy(pSatName, "IV");
 DrawJSat(x4, y4, d, u4c, A_NORMAL, pSatName);
 DrawJSatShadow(r4, d, (u4c - psi), de);

 // Displaying informations
 DrawStr(SCRX2,(SCR_Y-10),"N",A_XOR); 	// North
 DrawStr(SCRX2,10,"S",A_XOR); 							// South
 DrawStr(10,SCRY2,"E",A_XOR);								// East
 DrawStr((SCR_X-10),SCRY2,"W",A_XOR);				// West

 DrawStr(2,(SCR_Y-6 ),"IV: Calisto",A_XOR);
 DrawStr(2,(SCR_Y-12),"III: Ganymede",A_XOR);
 DrawStr(2,(SCR_Y-18),"II: Europa",A_XOR);
 DrawStr(2,(SCR_Y-24),"I: Io",A_XOR);

 sprintf(buffer,"\x91: %.1f\"",appd);					 // apparent diameter
 DrawStr((SCR_X-40),(SCR_Y-17),buffer, A_XOR);
 sprintf(buffer,"Dist.: %.3f AU",delta);				// Earth distance - 1AU= 149597870,691 Km
 DrawStr((SCR_X-46),(SCR_Y-5),buffer, A_XOR);
// sprintf(buffer,"1\"=%0.1fkm",(JDIAMKM/appd));	// evaluated scale in kilometers - a simplified method
 sprintf(buffer,"1\"=%0.1fkm",((Deg2Rad(appd)/3600.0)*(_1AU*delta))/appd);	// evaluated scale in kilometers
 DrawStr((SCR_X-44),(SCR_Y-11),buffer, A_XOR);
 sprintf(buffer,"%02d/%02d/%d %02d:%02d:%02d UTC",ptime->UTYear,ptime->UTMonth,ptime->UTDay,ptime->UTHour,ptime->UTMin,ptime->UTSec);
 DrawStr((SCR_X-90),2,buffer, A_XOR);
/*
 sprintf(buffer,"Scr:%0.1f\"",appd*(SCR_X/(float)(JUPITERD*pparam->Zoom)));
 DrawStr((SCR_X-46),8,buffer, A_XOR);
*/
 ngetchx();
 
 FontSetSys(F_6x8);
 free(buffer);
 return(0);
}

/* ====================================================== */
/* = Function    : Drawing Jovians' Sat.                = */
/* = Input       : X,Y center, radix, correction, attrib= */
/* = Output      : 0 if ok else 1                       = */
/* ====================================================== */
short DrawJSat(float x, float y, float r, float u, short att, char *name)
{
 float JovR = r/2.0;
 short state = 1;

 // before drawing sat, it's necessary to test if
 // it's behind, or in front of Jupiter
 // U angle must be defined between PI/2 and 3PI/2
 // sqrt(x^2+y^2) equ Jovian_diameter/2 so 1 in abs value
 if( !( (sqrt(x*x+y*y) < 1.0) && (u > 90.0) && (u < 270.0)) )
 {
  // size relation beetween the sat separation and planet
  x *= JovR;
  y *= JovR;
 
  x = floor(x+(float)(LCD_WIDTH>>1)); 
  y = floor(y+(float)(LCD_HEIGHT>>1));

 	if( !(state = DrawStar((short)x, (short)y, att)) )
   DrawStr((short)(x+2), (short)(y-4),name,att);
 }
 return(state);
}

/* ====================================================== */
/* = Function    : Drawing GRS for Jupiter              = */
/* = Input       : Meridian                             = */
/* = Output      : -                                    = */
/* ====================================================== */
void DrawGRS(float meridian, float r)
{
	float x = 0.0,
	      y = 0.0;
	      
	x = sinD(meridian);
	x *= (r/2.0);
	if( !((meridian > 90.0) && (meridian < 270.0)) )
	{
	 x += (float)(LCD_WIDTH>>1);
   y += (float)(LCD_HEIGHT>>1);
   x = floor(x);
   // 8.0 value have been choosen somewhere arbitrary for the SYS II !
   y = floor(y-(r/8.0));
 	 // casting is better to prevent the putpixel routine
   DrawShadow((short)x, (short)y, A_XOR);
   DrawStar((short)x, (short)y, A_XOR);
  }
}

/* ====================================================== */
/* = Function    : JTransit                             = */
/* = Input       : Julian date                          = */
/* = Output      : state                                = */
/* ====================================================== */
short JTransit(float jd, struct PARAM *pparam)
{
 float JupM = 0.0,
 			 EqCtr = 0.0,
 			 Alpha = 0.0,
 			 cor = 0.0,
 			 cmII = 0.0;
 
 float DIFF = 0.0,
       TT = 0.0,
       H = 0.0,
       px = 0.0,
       py = 0.0;
 			 
 char *buffer = NULL; //output buffer
 // allocate memory for the buffer
 if ((buffer = (char *)calloc(15,sizeof(char))) == NULL)
	return(MSG_MEM);

//--- 
// calculate by a simplified method the Jovian meridian
// Based on formula and informations from Project Pluto
//---

 JupM  = (jd - 2455636.938) * 360.0 / 4332.89709;
 EqCtr = 5.55 * sinD(JupM);
 Alpha = (jd -2451870.628) * 360.0 / 398.884 - EqCtr;
 cor = 11.0 * sinD(Alpha) + 5.0 * cosD(Alpha) - 1.25 * cosD(JupM) - EqCtr;

 // Centered Meridians at midnight
 cmII  = NormDeg((181.62 + 870.1869147 * jd + cor ));

 // calculate the transit time of Red Spot
 DIFF = (pparam->GRS_Ref)-cmII;
 if(DIFF < 0.0) DIFF+=360.0;
 DrawStr(px,py,"RS Transit:",A_XOR);
 py+=8.0;
 for( px=0.0 ; px<3.0 ; px++,py+=8.0,DIFF+=360.0 )
 {
  TT = DIFF/870.27*24.0;
  H = Int_Float(TT);
  if(!(H>23.0))
  {
   sprintf(buffer,"> %02.0f:%02.0f",H,Int_Float(60.0*(TT-H)));
   DrawStr(6,py,buffer,A_XOR);
  }
 } 
 // restoring memory
 free(buffer);
 return(0);
}

/* ====================================================== */
/* = Function    : Drawing Shadows' Sat.                = */
/* = Input       : sat dist, diameter, sun angle, attrib= */
/* = Output      : state                                = */
/* ====================================================== */
short DrawJSatShadow(float r, float d, float usun, float de)
{
 float x = r*sinD(usun),
 			 y = -r*cosD(usun)*sinD(de);
 short JovR = (short)Int_Float(d)>>1;

 if( (sqrt(x*x+y*y) < (float)JovR) && (usun > 90.0) && (usun < 270.0) )
 	return(1); 	
 else if( ( (x < -1.0) || (x > 1.0) ) )
  return(1);
 else if( ( (y < -1.0) || (y > 1.0) ) )
  return(1);

 // size relation beetween the sat separation and planet
 x *= (float)JovR;
 y *= (float)JovR;
 
 x = floor(x+(float)(LCD_WIDTH>>1)); 
 y = floor(y+(float)(LCD_HEIGHT>>1));
 
 DrawShadow((short)x, (short)y, A_NORMAL);
 
 return(0);
}