/**********************************************************/
/*                        T I A R E                       */
/* Astronomical Ressources & Ephemeris for TI Calculators */
/*               (c) 2003-2005, Yann DUCHEMIN             */
/* ------------------------------------------------------ */
/* This file is part of TIARE.                            */
/*                                                        */
/* TIARE is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public    */
/* License as published by the Free Software Foundation;  */
/* either version 2 of the License                        */
/*                                                        */
/* TIARE is distributed in the hope that it will be       */
/* useful, but WITHOUT ANY WARRANTY; without even the     */
/* implied warranty of MERCHANTABILITY or FITNESS FOR A   */
/* PARTICULAR PURPOSE. See the GNU General Public         */
/* License for more details.                              */
/*                                                        */
/* You should have received a copy of the GNU General     */
/* Public License along with TIARE; if not, write to      */
/* the Free Software Foundation, Inc., 59 Temple Place,   */
/* Suite 330, Boston, MA  02111-1307  USA                 */
/* ------------------------------------------------------ */
/* File Name : mars.c                                     */
/* Created   : 18 V   2003                                */
/* Modified  : 19 VII 2005                                */
/**********************************************************/

#include "mars.h"

/* ====================================================== */
/* = Function    : Mars appearence                      = */
/* = Input       : TIME & Observer structures           = */
/* = Output      : state                                = */
/* ====================================================== */
short MarsView(struct TIME *ptime, struct PLACE *pobs, struct PARAM *pparam)
{
 char *buffer = NULL; //output buffer
 short SCR_X = LCD_WIDTH,
 			 SCR_Y = LCD_HEIGHT,
 			 SCRX2 = SCR_X>>1,
 			 SCRY2 = SCR_Y>>1;

 float jd = 0.0;
 
 // allocate memory for the buffer
 if ((buffer = (char *)calloc(60,sizeof(char))) == NULL)
	return(MSG_MEM);

 float d  = 0.0,
       b  = 0.0,
       c  = 0.0,
       a1 = 0.0,
       d1 = 0.0,
       d9 = 0.0,
       n9 = 0.0,
       b4 = 0.0,
       bb = 0.0,
       t  = 0.0,
       l0 = 0.0,
       g0 = 0.0,
       l4 = 0.0,
       g4 = 0.0,
       f4 = 0.0,
       l5 = 0.0,
       a5 = 0.0,
       d5 = 0.0,
       g5 = 0.0,
       g6 = 0.0,
       l  = 0.0,
       l1 = 0.0,
       r  = 0.0,
       m  = 0.0,
       e  = 0.0,
       r4 = 0.0,
       v  = 0.0,
       s  = 0.0,
       k  = 0.0,
       sp = 0.0,
       cp = 0.0,
       pa = 0.0,
       k4 = 0.0,
       ll = 0.0,
       ad = 0.0,
       rr = 0.0;
 
// calculate the Julian Day
 jd = YMD2JD(ptime, REALTIME);	// REALTIME or MIDNIGHT
 d = (jd-2451545.0)+(65.0/86400.0);
 t = d/36525.0+1;
 
 l0 = 0.779072 + 0.00273790931 * d;
 g0 = 0.993126 + 0.0027377785  * d;
 l4 = 0.987353 + 0.00145575328 * d;
 g4 = 0.053856 + 0.00145561327 * d;
 f4 = 0.849694 + 0.00145569465 * d;
 l5 = 0.089608 + 0.00023080893 * d;
 g5 = 0.056531 + 0.00023080893 * d;
 g6 = 0.882987 + 9.294371E-05  * d;
 l0 = (l0-Int_Float(l0)) * (2*PI);
 g0 = (g0-Int_Float(g0)) * (2*PI);
 l4 = (l4-Int_Float(l4)) * (2*PI);
 g4 = (g4-Int_Float(g4)) * (2*PI);
 f4 = (f4-Int_Float(f4)) * (2*PI);
 l5 = (l5-Int_Float(l5)) * (2*PI);
 g5 = (g5-Int_Float(g5)) * (2*PI);
 g6 = (g6-Int_Float(g6)) * (2*PI);
  
 // sun
 l = 6910 * sin(g0) + 72 * sin(2*g0) - 17 * t * sin(g0) - 7 * cos(g0-g5);
 r = 1.00014 - 0.01675 * cos(g0) - 0.00014 * cos(2*g0);
 l = l0 + l / 206264.8;

 // mars
 m = 38451 * sin(g4) + 2238 * sin(2*g4) + 181 * sin(3*g4) - 52 * sin(2*f4);
 m = m + 37 * t * sin(g4) - 22 * cos(g4-2*g5) - 19 * sin(g4-g5) + 17 * cos(g4-g5);
 m = m + 17 * sin(4*g4) - 16 * cos(2*g4-2*g5) + 13 * cos(g0-2*g4);
 m = m - 10 * sin(g4-2*f4) + 7 * cos(g0-g4) - 7 * cos(2*g0-3*g4);
 b = 6603 * sin(f4) + 622 * sin(g4-f4) + 615 * sin(g4+f4) + 64 * sin(2*g4+f4);
 r4 = 1.53031 - 0.1417 * cos(g4) - 0.0066 * cos(2*g4) - 0.00047 * cos(3*g4);
 l4 = l4 + m / 206264.8;
 b4 = b / 206264.8;
 n9 = r4 * cos(b4) * sin(l4-l);
 d9 = r4 * cos(b4) * cos(l4-l) + r;
 l1 = (atan2(n9,d9));
 if (d9 < 0.0) l1 += PI;
 ll = (l1 + l);
 v = (n9 * n9 + d9 * d9);
 s = (r4 * sin(b4));
 rr = sqrt(v+s*s);
 s = (s / rr);
 bb = (atan2(s,sqrt(1-s*s)));
 e = (84428 - 47 * t) / 206264.8;
 n9 = (sin(ll) * cos(e) - tan(bb) * sin(e));
 d9 = (cos(ll));
 a5 = (atan2(n9,d9));
 if (d9 < 0.0) a5 += PI;
 s = sin(bb) * cos(e) + cos(bb) * sin(e) * sin(ll);
 d5 = (atan2(s,sqrt(1-s*s)));
 t = t - 1;
 a1 = (Deg2Rad(317.681 + 0.678 * t));
 d1 = (Deg2Rad(52.886 + 0.352 * t));
 d9 = (cos(d5) * sin(a1-a5));
 n9 = (sin(d1) * cos(d5) * cos(a1-a5) - cos(d1) * sin(d5));
 k  = Rad2Deg(atan2(n9,d9));
 if (d9 < 0.0) k += 180.0;
 c = Norm180(176.655 + 350.8919830 * (d - 0.0057755 * rr) + 0.62 * t - k);
 sp = (cos(d1) * sin(a1-a5));
 cp = (sin(d1) * cos(d5) - cos(d1) * sin(d5) * cos(a1-a5));
 pa =  Rad2Deg(atan2(sp,cp));

 if (cp < 0.0) pa += 180.0;
 if (pa < 0.0) pa += 360.0;
 if (pa >= 360.0) pa -= 360.0;
 k4 = (pow(r4+rr,2) - r * r)/(4 * r4 * rr);
    
 ad = MDIAMSEC / rr;

 DrawStr(SCRX2-3,3,"S",A_NORMAL);
 FontSetSys(F_4x6);
 sprintf(buffer,"\x62 : %.1f",c+0.5);
 DrawStr(2,(SCR_Y-6 ),buffer,A_XOR);
 sprintf(buffer,"\x90 : %.1f%%",k4*100+0.5);
 DrawStr(2,(SCR_Y-12),buffer,A_XOR);
 sprintf(buffer,"\x9F : %.1f",pa+0.5);
 DrawStr(2,(SCR_Y-18),buffer,A_XOR);
 
 sprintf(buffer,"\x91: %.1f\"",ad);						// apparent diameter
 DrawStr((SCR_X-40),(SCR_Y-17),buffer, A_XOR);
 sprintf(buffer,"Dist.: %.3f AU",rr);		// Earth distance 
 DrawStr((SCR_X-50),(SCR_Y-5),buffer, A_XOR);
 sprintf(buffer,"1\"=%0.1fkm",((Deg2Rad(ad)/3600.0)*(_1AU*rr))/(ad));	// evaluated scale in kilometers - a simplified method
 DrawStr((SCR_X-44),(SCR_Y-11),buffer, A_XOR);
 sprintf(buffer,"%02d/%02d/%d %02d:%02d:%02d UTC",ptime->UTYear,ptime->UTMonth,ptime->UTDay,ptime->UTHour,ptime->UTMin,ptime->UTSec);
 DrawStr((SCR_X-90),2,buffer, A_XOR);
 FontSetSys(F_6x8);
 
 free(buffer); 
 
 // Zoom Corrections
  ad = (float)(MARSD*pparam->Zoom);
  if( Int_Float(ad) < (float)SCRX2 )
   DrawSphere(SCRX2, SCRY2, (unsigned short)Int_Float(ad), Deg2Rad((180.0*(k4*100.0+0.5))/100.0), Deg2Rad(360.0-(pa+0.5)), 1.5708);

 ngetchx();
 return(0);
}
