/**********************************************************/
/*                        T I A R E                       */
/* Astronomical Ressources & Ephemeris for TI Calculators */
/*               (c) 2003-2005, Yann DUCHEMIN             */
/* ------------------------------------------------------ */
/* This file is part of TIARE.                            */
/*                                                        */
/* TIARE is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public    */
/* License as published by the Free Software Foundation;  */
/* either version 2 of the License                        */
/*                                                        */
/* TIARE is distributed in the hope that it will be       */
/* useful, but WITHOUT ANY WARRANTY; without even the     */
/* implied warranty of MERCHANTABILITY or FITNESS FOR A   */
/* PARTICULAR PURPOSE. See the GNU General Public         */
/* License for more details.                              */
/*                                                        */
/* You should have received a copy of the GNU General     */
/* Public License along with TIARE; if not, write to      */
/* the Free Software Foundation, Inc., 59 Temple Place,   */
/* Suite 330, Boston, MA  02111-1307  USA                 */
/* ------------------------------------------------------ */
/* File Name : menu.c                                     */
/* Created   : 27 II  2003                                */
/* Modified  : 22 XII 2005                                */
/**********************************************************/

#include "menu.h"

/* ====================================================== */
/* = Function    : Main menu                            = */
/* = Input       : Time and Local Observer parameters   = */
/* = Output      : Error code (short)                   = */
/* ====================================================== */
short MainMenu(struct TIME *ptime, struct PLACE *pplace, struct PARAM *pparam)
{
 char d[]="::: T I A R E :::";
 short int err_code = 0;
 HANDLE FirstMenu;
	
	FirstMenu = PopupNew (d, 0);
	// Major
	PopupAddText (FirstMenu, 0, "Parameters", 1);
  PopupAddText (FirstMenu, -1, "Calendar", 2);
  PopupAddText (FirstMenu, 0, "Ephemeris", 3);
	PopupAddText (FirstMenu, 0, "Graphic View", 4);
  PopupAddText (FirstMenu, -1, "Exit", 6);
  // SubMajor
	PopupAddText (FirstMenu, 1, "Time", 10);
	PopupAddText (FirstMenu, 1, "Location", 11);
	PopupAddText (FirstMenu, 1, "Tools", 12);
//	PopupAddText (FirstMenu, 1, "New Object", 13);
//	PopupAddText (FirstMenu, 1, "Edit Object", 14);
	
	PopupAddText (FirstMenu, 3, "Sun", 30);
	PopupAddText (FirstMenu, 3, "Moon", 31);
	PopupAddText (FirstMenu, 3, "Mercury", 32);
	PopupAddText (FirstMenu, 3, "Venus", 33);
	PopupAddText (FirstMenu, 3, "Mars", 34);
  PopupAddText (FirstMenu, 3, "Jupiter", 35);
	PopupAddText (FirstMenu, 3, "Saturn", 36);
	
	PopupAddText (FirstMenu, 4, "Moon", 40);
	PopupAddText (FirstMenu, 4, "Jovian's sat.", 41);
  PopupAddText (FirstMenu, 4, "Saturn view", 42);
  PopupAddText (FirstMenu, 4, "Mars view", 43);
  PopupAddText (FirstMenu, 4, "Sky Map", 44);
	
	while(err_code != -1)
	{
	 clrscr();							// Clear Screen
	 DrawScrBorder();				// Make a decoration ;o)
	 DrawCopyright();
   err_code = PopupDo(FirstMenu, CENTER, CENTER, 0); // catch menu level
   GKeyFlush();
	 clrscr();
	 switch (err_code)
	 {
		// Parameters menu  	
	   case 10:
			if((err_code = EdTime(ptime, pplace->DTime)) != 0)
	      ErrMsg(MSG_ERR, err_code);
		 break;
		 case 11:
	    if((err_code = EdPlace(pplace)) != 0)
	      ErrMsg(MSG_ERR, err_code);
	   break;
		 case 12:
	    if((err_code = EdParam(pparam)) != 0)
	      ErrMsg(MSG_ERR, err_code);
	   break;
		// Calendar menu
		 case 2:
		  if((err_code = Calend(ptime, pplace)) != 0)
		    ErrMsg(MSG_INFO, MSG_NIMP);
	   break;
		// Ephemeris menu
		 case 30:
		  if((err_code = DrawingPosObj(SUN, ptime, pplace, pparam)) != 0)
		    ErrMsg(MSG_INFO, MSG_NIMP);
	   break;
		 case 31:
		  if((err_code = DrawingPosObj(MOON, ptime, pplace, pparam)) != 0)
		    ErrMsg(MSG_INFO, MSG_NIMP);
	   break;
		 case 32:
		  if((err_code = DrawingPosObj(MERCURY, ptime, pplace, pparam)) != 0)
		    ErrMsg(MSG_INFO, MSG_NIMP);
	   break;
		 case 33:
		  if((err_code = DrawingPosObj(VENUS, ptime, pplace, pparam)) != 0)
		    ErrMsg(MSG_INFO, MSG_NIMP);
	   break;
		 case 34:
		  if((err_code = DrawingPosObj(MARS, ptime, pplace, pparam)) != 0)
		    ErrMsg(MSG_INFO, MSG_NIMP);
	   break;   
		 case 35:
		  if((err_code = DrawingPosObj(JUPITER, ptime, pplace, pparam)) != 0)
		    ErrMsg(MSG_INFO, MSG_NIMP);
	   break;
		 case 36:
		  if((err_code = DrawingPosObj(SATURN, ptime, pplace, pparam)) != 0)
		    ErrMsg(MSG_INFO, MSG_NIMP);
	   break; 
		// Graphical menu
		 case 40:
		  if((err_code = DrawMoon(ptime, pplace)) != 0)
		    ErrMsg(MSG_INFO, MSG_NIMP);
	   break;
		 case 41:
		  if((err_code = JSat(ptime, pplace, pparam)) != 0)
		    ErrMsg(MSG_INFO, MSG_NIMP);
	   break; 
		 case 42:
		  if((err_code = RingSat(ptime, pplace, pparam)) != 0)
	    ErrMsg(MSG_INFO, MSG_NIMP);
	   break; 
		 case 43:
		  if((err_code = MarsView(ptime, pplace, pparam)) != 0)
	    ErrMsg(MSG_INFO, MSG_NIMP);
	   break; 
		 case 44:
			DrawSky();
		 break; 
		// EXIT menu  
		 case 6:
			HeapFree(FirstMenu);
			err_code = -1;			//	exit(0);
	   break;
	  }
  }
 return(err_code);
}

/* ====================================================== */
/* = Function    : Time Edition                         = */
/* = Input       : time struct, Time Zone               = */
/* = Output      : state                                = */
/* ====================================================== */
short EdTime(struct TIME *time, short TZ)
{
 char *buffer = NULL;
 HANDLE Popup = H_NULL;

 // allocate memory for the buffer
 if ((buffer = (char *)calloc(21,sizeof(char))) == NULL)
	return(MSG_MEM);
 // allocate memory for the dialog box
 else if ((Popup = DialogNewSimple (157, 70)) == H_NULL)
 {
	// free input buffer memory before exiting
	free(buffer);		
	return(MSG_MEM);
 }

 if( !(RFchTime(time)) ) // take old values if ficht already exist
 {
  // format datas left centered (- opt) and force unsigned
  sprintf(buffer,"%-3hu%-3hu%-5hu%-3hu%-3hu%-3hu",time->ObsDay,time->ObsMonth,time->ObsYear,time->ObsHour,time->ObsMin,time->ObsSec);
  buffer[2] = 0;  // to format the dialog request with end of line
  buffer[5] = 0;
  buffer[10] = 0;
  buffer[13] = 0;
  buffer[16] = 0;
  buffer[19] = 0;
 }
 
 DialogAddTitle (Popup, "Observation's Time", BT_OK, BT_CANCEL);
 DialogAddRequest (Popup, 5, 20, "DD  :", 0, 2, 3);
 DialogAddRequest (Popup, 5, 30, "MM  :", 3, 2, 3);
 DialogAddRequest (Popup, 5, 40, "YYYY:", 6, 4, 5);
 DialogAddRequest (Popup, 85, 20, "HH :", 11, 2, 3);
 DialogAddRequest (Popup, 85, 30, "MM :", 14, 2, 3);
 DialogAddRequest (Popup, 85, 40, "SS :", 17, 2, 3);
 if( DialogDo (Popup, CENTER, CENTER, buffer, NULL) == KEY_ENTER)
 {
  time->ObsDay   = atoi(buffer);
  time->ObsMonth = atoi(buffer+3);
  time->ObsYear  = atoi(buffer+6);
  time->ObsHour  = atoi(buffer+11);
  time->ObsMin   = atoi(buffer+14);
  time->ObsSec   = atoi(buffer+17); 	
 }
 HeapFree (Popup); 
 free(buffer); // buffer not needed more
 
 if((VerifyTimeStruct(time)) == 0) // verify data entered before save
 	return(FchTime(time, TZ));
 else return(RFchTime(time));
}

/* ====================================================== */
/* = Function    : Site's Observer Edition              = */
/* = Input       : site struct                          = */
/* = Output      : state                                = */
/* ====================================================== */
short EdPlace(struct PLACE *place)
{
 char *buffer = NULL;
 HANDLE Popup = H_NULL; 

 // allocate memory for the buffer
 if ((buffer = (char *)calloc(32,sizeof(char))) == NULL)
	return(MSG_MEM);
 // allocate memory for the dialog box
 else if ((Popup = DialogNewSimple (157, 85)) == H_NULL)
 {
	// free input buffer memory before exiting
	free(buffer);		
	return(MSG_MEM);
 }
 
 if( !(RFchLocal(place)) ) // take old values if ficht already exist
 {
  // format datas left centered (- opt) and force signed short integer
  sprintf(buffer,"%-4hd%-3hu%-3hu%-5hd%-3hu%-3hu%-6hd%-4hd",place->LatDeg,place->LatMin,place->LatSec,place->LongDeg,place->LongMin,place->LongSec,place->Alt,place->DTime);
  buffer[3] = 0;  // to format the dialog request with end of line
  buffer[6] = 0;
  buffer[9] = 0;
  buffer[14] = 0;
  buffer[17] = 0;
  buffer[20] = 0;
  buffer[26] = 0;
  buffer[30] = 0;
 }

 DialogAddTitle (Popup, "Localisation", BT_OK, BT_CANCEL);
 DialogAddText (Popup,  5, 15, "Latitude :");
  DialogAddRequest (Popup,  5, 25, "D", 0, 3, 4);
  DialogAddRequest (Popup, 52, 25, "M", 4, 2, 3);
  DialogAddRequest (Popup, 99, 25, "S", 7, 2, 3);
 DialogAddText (Popup,  5, 35, "Longitude :");
  DialogAddRequest (Popup,  5, 45, "D", 10, 4, 5);
  DialogAddRequest (Popup, 52, 45, "M", 15, 2, 3);
  DialogAddRequest (Popup, 99, 45, "S", 18, 2, 3);
 DialogAddRequest (Popup, 5, 57, "Alt :", 21, 5, 6);
 DialogAddRequest (Popup, 81, 57, "GMT :", 27, 3, 4);

 if( DialogDo (Popup, CENTER, CENTER, buffer, NULL) == KEY_ENTER)
 {
 	place->LatDeg = atoi(buffer);
  place->LatMin = atoi(buffer+4);
  place->LatSec = atoi(buffer+7);
  place->LongDeg  = atoi(buffer+10);
  place->LongMin  = atoi(buffer+15);
  place->LongSec  = atoi(buffer+18);
  place->Alt     = atoi(buffer+21);
  place->DTime   = atoi(buffer+27); 
 }

 HeapFree (Popup); 
 free(buffer); // buffer not needed more

 if((VerifyPlaceStruct(place)) == 0) // verify data entered before save
 	return(FchLocal(place));
 else return(RFchLocal(place));
}

/* ====================================================== */
/* = Function    : Extras' Parameters                   = */
/* = Input       : param struct                         = */
/* = Output      : state                                = */
/* ====================================================== */
short EdParam(struct PARAM *pparam)
{
 char *buffer = NULL;
  
 HANDLE Popup = H_NULL; 
 HANDLE HMount = H_NULL;
 HANDLE HProto = H_NULL;
 
 // allocate memory for the buffer
 if ((buffer = (char *)calloc(9,sizeof(char))) == NULL) 
 	return(MSG_MEM);
 // allocate memory for the dialog box
 else if ((Popup = DialogNewSimple (157, 85)) == H_NULL)
 {
	// free input buffer memory before exiting
	free(buffer);		
	return(MSG_MEM);
 }
 
 if( !(RFchParam(pparam)) ) // take old values if ficht already exist
 {
  sprintf(buffer,"%-4hu%-3hu",pparam->GRS_Ref,pparam->Zoom);
  buffer[3] = 0;	// to format the dialog request with end of line
  buffer[6] = 0;
 }
 DialogAddTitle (Popup, "Config", BT_OK, BT_CANCEL);
 DialogAddRequest (Popup, 5, 15, "GRS Ref. :", 0, 3, 4);
 DialogAddRequest (Popup, 5, 27, "Zoom Factor :", 4, 2, 3);
 
 if( (HMount = PopupNew(NULL, 0)) != H_NULL ) // NULL for no title & 0 to autocalculate weight
 { 
  PopupAddText( HMount, -1, "Equ", 1); 
  PopupAddText( HMount, -1, "Altaz", 2);
//  PopupAddText( HMount, -1, "German", 3);
   DialogAddPulldown(Popup, 5, 39, "Mount :", HMount, 0);
 }
 
 if( (HProto = PopupNew(NULL, 0)) != H_NULL ) // NULL for no title & 0 to autocalculate weight
 { 
  PopupAddText( HProto, -1, "None", 1); 
  PopupAddText( HProto, -1, "LX200", 2);
//  PopupAddText( HProto, -1, "Ascom", 3);
   DialogAddPulldown(Popup, 5, 52, "Protol :", HProto, 1);
 }
 
 if (DialogDo (Popup, CENTER, CENTER, buffer, pparam->PullDownBuffer) == KEY_ENTER)
 {
  pparam->GRS_Ref = atoi(buffer);
  pparam->Zoom    = atoi(buffer+4);
 }
 
 HeapFree (HMount);
 HeapFree (HProto);
 HeapFree (Popup); 
 free(buffer); // buffer not needed more
 return(FchParam(pparam));
}
