/**********************************************************/
/*                        T I A R E                       */
/* Astronomical Ressources & Ephemeris for TI Calculators */
/*               (c) 2003-2005, Yann DUCHEMIN             */
/* ------------------------------------------------------ */
/* This file is part of TIARE.                            */
/*                                                        */
/* TIARE is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public    */
/* License as published by the Free Software Foundation;  */
/* either version 2 of the License                        */
/*                                                        */
/* TIARE is distributed in the hope that it will be       */
/* useful, but WITHOUT ANY WARRANTY; without even the     */
/* implied warranty of MERCHANTABILITY or FITNESS FOR A   */
/* PARTICULAR PURPOSE. See the GNU General Public         */
/* License for more details.                              */
/*                                                        */
/* You should have received a copy of the GNU General     */
/* Public License along with TIARE; if not, write to      */
/* the Free Software Foundation, Inc., 59 Temple Place,   */
/* Suite 330, Boston, MA  02111-1307  USA                 */
/* ------------------------------------------------------ */
/* File Name : moon.c                                     */
/* Created   : 27 II   2003                               */
/* Modified  : 06 V    2005                               */
/**********************************************************/

#include "moon.h"

/* ====================================================== */
/* = Function    : Drawing Moon apparence               = */
/* = Input       : TIME, PLACE									        = */
/* = Output      : state    		                        = */
/* ====================================================== */
short DrawMoon(struct TIME *ptime, struct PLACE *pobs)
{
 char *buffer = NULL; //output buffer
 short SCR_X = LCD_WIDTH,
 			 SCR_Y = LCD_HEIGHT,
 			 SCRX2 = SCR_X>>1,
 			 SCRY2 = SCR_Y>>1;

 float LocalST = 0.0;
 float jd = 0.0,
 				ip = 0.0,
 				ag = 0.0;

 struct OBJECT Planet;
 struct OBJECT *pplanet;
 pplanet = &Planet;
 
 // allocate memory for the buffer
 if ((buffer = (char *)calloc(60,sizeof(char))) == NULL)
	return(MSG_MEM);

 // calculate the Julian Day
 jd = YMD2JD(ptime, REALTIME);	// REALTIME or MIDNIGHT
 
 // calculate illumination phase
 ip = NormExp((jd-2451550.1) / M_SYNODIC);
 ag = ip*M_SYNODIC; // moon age in days
 ip *= 2.0*PI;			// convert phase to radians

 LocalST = RSTG_H(STG_H(YMD2JD(ptime, REALTIME)), DMS2Float(pobs->LongDeg, pobs->LongMin, pobs->LongSec), ptime);

 PosObj(LocalST, ptime, pobs, pplanet, MOON); // compute for the Moon

 DrawStr(SCRX2-3,3,"N",A_NORMAL);
 FontSetSys(F_4x6);
 sprintf(buffer,"Age: %0.1f",ag);
 DrawStr(5,5,buffer, A_XOR);
 sprintf(buffer,"\x92:%0.1f",Rad2Deg(ip));
 DrawStr(5,12,buffer, A_XOR);
 sprintf(buffer,"\x91: %.1f\'",MOONDIAMSEC/pplanet->DE);					 // apparent diameter
 DrawStr((SCR_X-40),(SCR_Y-17),buffer, A_XOR);
 sprintf(buffer,"Dist.: %.0f Km",pplanet->DE*MOONDIAMIN);				// Earth distance 
 DrawStr((SCR_X-56),(SCR_Y-5),buffer, A_XOR);
 sprintf(buffer,"1\"=%0.2fkm",((MOONDIAMIN/(MOONDIAMIN/pplanet->DE))/60.0));	// evaluated scale in kilometers - a simplified method
 DrawStr((SCR_X-44),(SCR_Y-11),buffer, A_XOR);
 sprintf(buffer,"%02d/%02d/%d %02d:%02d:%02d UTC",ptime->UTYear,ptime->UTMonth,ptime->ObsDay,ptime->UTHour,ptime->UTMin,ptime->UTSec);
 DrawStr((SCR_X-90),2,buffer, A_XOR);
 FontSetSys(F_6x8);
 free(buffer); 
 // 0.061 is choosen from a medium moon's lib instead of compute it a this time
 // 1.571 (PI/2) is the start point for X projection, need to add the angle to this
 DrawSphere(SCRX2, SCRY2, 40, ip, 0.061, 1.5708);
 
 ngetchx();
 return(0);
}