/**********************************************************/
/*                        T I A R E                       */
/* Astronomical Ressources & Ephemeris for TI Calculators */
/*               (c) 2003-2005, Yann DUCHEMIN             */
/* ------------------------------------------------------ */
/* This file is part of TIARE.                            */
/*                                                        */
/* TIARE is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public    */
/* License as published by the Free Software Foundation;  */
/* either version 2 of the License                        */
/*                                                        */
/* TIARE is distributed in the hope that it will be       */
/* useful, but WITHOUT ANY WARRANTY; without even the     */
/* implied warranty of MERCHANTABILITY or FITNESS FOR A   */
/* PARTICULAR PURPOSE. See the GNU General Public         */
/* License for more details.                              */
/*                                                        */
/* You should have received a copy of the GNU General     */
/* Public License along with TIARE; if not, write to      */
/* the Free Software Foundation, Inc., 59 Temple Place,   */
/* Suite 330, Boston, MA  02111-1307  USA                 */
/* ------------------------------------------------------ */
/* File Name : pict.c                                     */
/* Created   : 27 II  2003                                */
/* Modified  : 21 VII 2005                                */
/**********************************************************/

#include "pict.h"

/* ====================================================== */
/* = Function    : Make a box arround th screen         = */
/* = Input       : -                                    = */
/* = Output      : -                                    = */
/* ====================================================== */
void DrawScrBorder(void)
{
	DrawLine(0,0,0,LCD_HEIGHT-8,A_NORMAL);
	DrawLine(0,0,LCD_WIDTH-1,0,A_NORMAL);
	DrawLine(LCD_WIDTH-1,0,LCD_WIDTH-1,LCD_HEIGHT-8,A_NORMAL);
	DrawLine(LCD_WIDTH-1,LCD_HEIGHT-8,0,LCD_HEIGHT-8,A_NORMAL);
}

/* ====================================================== */
/* = Function    : Print more information about TIARE   = */
/* = Input       : -                                    = */
/* = Output      : -                                    = */
/* ====================================================== */
void DrawCopyright(void)
{
 FontSetSys(F_4x6);
 DrawStr(1,45,(char *)intro[VERS0],A_XOR);
 DrawStr(6,52,(char *)intro[VERS1],A_XOR);
 DrawStr(LCD_WIDTH-140,LCD_HEIGHT-(LCD_HEIGHT-4),(char *)intro[CONTENT],A_XOR);
 DrawStr(LCD_WIDTH-110,LCD_HEIGHT-(LCD_HEIGHT-10),(char *)intro[COPYRIGHT],A_XOR);
 // URL=32car.*4=132pix - 1car.*6=6 +1 pixel spc
 DrawStr(LCD_WIDTH-132,LCD_HEIGHT-16,(char *)intro[URL],A_XOR);
 FontSetSys(F_6x8);
}

/* ====================================================== */
/* = Function    : Fast Drawing small Circle function   = */
/* = Input       : X,Y center & ray                     = */
/* = Output      : -                                    = */
/* ====================================================== */
void DrawCircle(unsigned short x, unsigned short y, unsigned int r)
{
 short int c = r>>1,
			     cx = 0,
			     cy = r>>1,
			     cz = 0;
/*
 if(( (x+r) > (LCD_WIDTH-1)) || ( (y+r) > (LCD_HEIGHT-1)))
 { cy=0; }
 */
 do
 {
  DrawPix( x+cx, y+cy, A_NORMAL);
  DrawPix( x-cx, y+cy, A_NORMAL);
  DrawPix( x-cx, y-cy, A_NORMAL);
  DrawPix( x+cx, y-cy, A_NORMAL);
  DrawPix( x+cy, y+cx, A_NORMAL);
  DrawPix( x-cy, y+cx, A_NORMAL);
  DrawPix( x-cy, y-cx, A_NORMAL);
	DrawPix( x+cy, y-cx, A_NORMAL);
	c += cx++;
	if( (c += cx) >= 0 )
	{
	 cz = cy--;
	 c -= cz+cy;
	}
 }
 while( cx <= cy );
}

/* ====================================================== */
/* = Function    : Drawing Star function                = */
/* = Input       : X,Y center & attribute               = */
/* = Output      : 0 if ok else 1                       = */
/* ====================================================== */
short DrawStar(unsigned short x, unsigned short y, unsigned short att)
{
 // x & y are unsigned because the screen size couldn't be negative
 // it's shorter than a test like x<0 every true
 if((x > (LCD_WIDTH-1)) || (y > (LCD_HEIGHT-1)))
 	return(1);
 DrawPix(x-1, y, att);
 DrawPix(x, y-1, att);
 DrawPix(x+1, y, att);
 DrawPix(x, y+1, att);
 return(0);
}

/* ====================================================== */
/* = Function    : Drawing Planets' positions           = */
/* = Input       : object number, TIME & PLACE struct   = */
/* = Output      : /  					                        = */
/* ====================================================== */
short DrawingPosObj(short obj, struct TIME *ptime, struct PLACE *pobs, struct PARAM *pparam)
{
 short xs = LCD_WIDTH>>4,			// number of columns
 			 ys = LCD_HEIGHT>>4;		// number of lines
 			 
 short err = 0;
 float LocalST = 0.0,
 			 diameter = 0.0;

 struct OBJECT Planet;
 struct OBJECT *pplanet;
 pplanet = &Planet;
 
 struct DMS Position;		// define a structure for the position conversions
 struct DMS *pposition;	// define a structure pointer
 pposition = &Position;	// initialize ptr 	
 		 
 char *buffer = NULL; //output buffer
 // allocate memory for the buffer
 if ((buffer = (char *)calloc(30,sizeof(char))) == NULL)
	return(MSG_MEM);

 //--- Display results
 FontSetSys(F_4x6);
 DrawStr(xs,ys*14,"LST:",A_NORMAL); 
 
 LocalST = RSTG_H(STG_H(YMD2JD(ptime, REALTIME)), DMS2Float(pobs->LongDeg, pobs->LongMin, pobs->LongSec), ptime);

 Float2DMS(LocalST,pposition);
 sprintf(buffer,"%02.0fh%02.0fm%02.0fs",pposition->d,pposition->m,pposition->s);
 DrawStr((xs<<1),ys*14,buffer,A_NORMAL);
 
 DrawStr(xs,ys*15,"UTC:",A_NORMAL); 
 sprintf(buffer,"%d/%02d/%02d %02d:%02d:%02d",ptime->UTYear,ptime->UTMonth,ptime->ObsDay,ptime->UTHour,ptime->UTMin,ptime->UTSec);
 DrawStr((xs<<1),ys*15,buffer, A_XOR);
 FontSetSys(F_6x8);
 
 // computing some interested informations 
 // --------------------------------------
 PosObj(LocalST, ptime, pobs, pplanet, obj);
 
 switch (obj)
 {
 	case SUN:
   diameter = SUNDIAMSEC;
  break;
 	case MOON:
   diameter = MOONDIAMSEC;
  break;
  case MERCURY:
   diameter = MeDIAMSEC;
  break; 
 	case MARS:
   diameter = MDIAMSEC;
  break;
 	case VENUS:
   diameter = VDIAMSEC;
  break;
 	case SATURN:
   diameter = SDIAMSEC;
  break;
  case JUPITER:
   diameter = JDIAMSEC;
  break;
 }
 
 if( obj != MOON)
 { 
  // Sun distance
  DrawStr(xs<<3,ys,"Sun Dist.",A_NORMAL); 
  sprintf(buffer,"%.4f AU",pplanet->DS);
  DrawStr(xs<<3,ys*2,buffer,A_NORMAL);
  // Earth distance
 	sprintf(buffer,"%.4f AU",pplanet->DE);
 }
 else
 {
  sprintf(buffer,"%.0f Km",(pplanet->DE*MOONDIAMIN));
 }
 DrawStr(xs,ys,"Earth Dist.",A_NORMAL); 
 DrawStr(xs,ys*2,buffer,A_NORMAL);
 
 // print the Right Ascension
 DrawStr(xs<<3,ys*4,"\x80",A_NORMAL); 
 Float2DMS(pplanet->Dec,pposition);
 sprintf(buffer,"%02.0f%02.0f'%02.0f\"",pposition->d,pposition->m,pposition->s);
 DrawStr(xs,ys*5,buffer,A_NORMAL);

 // print the Declinaison
 DrawStr(xs,ys*4,"\x85",A_NORMAL); 
 Float2DMS(pplanet->RA,pposition);
 sprintf(buffer,"%02.0fh%02.0fm%02.0fs",pposition->d,pposition->m,pposition->s);
 DrawStr(xs<<3,ys*5,buffer,A_NORMAL);

 // print altitude
 DrawStr(xs<<3,ys*7,"Alt.",A_NORMAL);
 Float2DMS(pplanet->Az,pposition);
 sprintf(buffer,"%02.0f%02.0f'%02.0f\"",pposition->d,pposition->m,pposition->s);
 DrawStr(xs,ys*8,buffer,A_NORMAL);

 // print azimuth
 DrawStr(xs,ys*7,"Az.",A_NORMAL); 
 Float2DMS(pplanet->Al,pposition);
 sprintf(buffer,"%02.0f%02.0f'%02.0f\"",pposition->d,pposition->m,pposition->s);
 DrawStr(xs<<3,ys*8,buffer,A_NORMAL);

 // print rise & set
 Float2DMS(pplanet->rise,pposition);
 sprintf(buffer,"Rise:%02.0fh%02.0f",pposition->d,pposition->m);
 DrawStr(xs,ys*10,buffer,A_NORMAL); 
 
 Float2DMS(pplanet->set,pposition);
 sprintf(buffer,"Set:%02.0fh%02.0f",pposition->d,pposition->m);
 DrawStr(xs<<3,ys*10,buffer,A_NORMAL);
 
 // print apparent angular diameter
 if(obj == SUN || obj == MOON)
  sprintf(buffer,"\x91:%0.1f'",diameter/pplanet->DE);
 else
  sprintf(buffer,"\x91:%0.1f\"",diameter/pplanet->DE);
 DrawStr(xs,ys*12,buffer,A_NORMAL);

 sprintf(buffer,"\x92:%0.1f",pplanet->Ph);
 DrawStr(xs<<3,ys*12,buffer,A_NORMAL);
  
  //--- End
 if( obj == JUPITER) 
 {
 	ngetchx();
  clrscr();
  err = JTransit(YMD2JD(ptime, MIDNIGHT), pparam);
 }
 // restoring memory
 free(buffer);
 ngetchx();
 return(err);
}

/* ====================================================== */
/* = Function    : Make a calendar's box on the screen  = */
/* = Input       : TIME struct                          = */
/* = Output      : 0 or err code                        = */
/* ====================================================== */
short DrawCalendar(struct TIME *ptime)
{
 char *buffer = NULL; //output buffer
 unsigned int h = (unsigned int)(LCD_HEIGHT / 10);
 unsigned int w = (unsigned int)(LCD_WIDTH / 7);
 float jd = DayOfWeek(YMD2JD(ptime, MIDNIGHT))-1;
 unsigned int c = 0, d = 0;
 unsigned int x = 2, y = (3*h)+2;
 unsigned int m = (unsigned int)NbDaysFMonth(ptime->ObsMonth, ptime->ObsYear);
 
  // allocate memory for the buffer
 if ((buffer = (char *)calloc(10,sizeof(char))) == NULL)
	return(MSG_MEM);

 // drawing days of week
 for(h+=h, c=LCD_WIDTH, d=6; c>w ;c-=w,d--)
  DrawStr((c-w),h+2,(char *)days[d],A_REVERSE);
  
 // search where printing the first day's month
 if( jd<0 )
  x += (6*w);
 else
  x += (w*jd);

 for(c=1 ; c<=m ; x=2,y+=h>>1)
 {
  for(d=x; d<LCD_WIDTH-13 && c<=m ; d+=w,x+=w,c++)
  {
   sprintf(buffer,"%02u",c);
   DrawStr(x+2, y, buffer,A_NORMAL);
  }
 }
 sprintf(buffer,"%s %hu",(char *)month[ptime->ObsMonth-1],ptime->ObsYear);
 DrawStr((LCD_WIDTH>>1)-20,10,buffer,A_NORMAL); 
 free(buffer);
return(0);
}

/* ====================================================== */
/* = Function    : Drawing Shadow function              = */
/* = Input       : X,Y center & attribute               = */
/* = Output      : 0 if ok else 1                       = */
/* ====================================================== */
short DrawShadow(unsigned short x, unsigned short y, unsigned short att)
{
 // x & y are unsigned because the screen size couldn't be negative
 // it's shorter than a test like x<0 every true
 DrawPix(x, y, att);
 DrawPix(x-1, y-1, att);
 DrawPix(x+1, y+1, att);
 DrawPix(x-1, y+1, att);
 DrawPix(x+1, y-1, att);
 return(0);
}

/* ====================================================== */
/* = Function    : Drawing Sphere                       = */
/* = Input       : X,Y, ray, Phase & angleY, angleX     = */
/* =             Angles are used for projection formula = */
/* = Output      : -                                    = */                                                                                                                                                                                                                                                                                                                                       
/* ====================================================== */
void DrawSphere(unsigned short x, unsigned short y, unsigned short r, float phase, float angleY, float angleX)
{
 char *buffer = NULL; //output buffer
 // real sphere coords
 float xa = 0.0,
 			 ya = 0.0,
 			 za = 0.0;
 // computation var.
 float	a = 0.0,
			  b = 0.0,
			  j = 0.0;
 // counter for latitudes
 short int latitude = 0,
     			 longitude = 0,
           delta = 0;
 short int vertex = 0;          
 // search the max vertex for the needed parameters
 //-----------------------------------------------------------
 if(phase>PI)
  {
 	 b=phase-PI;
   j=PI;
  }
  else
  {
 	 b=0.0;
 	 j=phase;
  }
 	// compute all the line	
  while(b<=j)
  {
   vertex++;
 	 b+=SPHERE_PRECISION;
 	}
 vertex*=(short)ceil(PI/SPHERE_PRECISION);
 // create a double dim tab for sphere & coords x & y
 short tab[vertex][2];
 //-----------------------------------------------------------
 // allocate memory for the buffer
 buffer = (char *)calloc(6,sizeof(char));

  // computing sphericals points
  for( a=-PI/2.0 ; a<=PI/2.0 ; a+=SPHERE_PRECISION,latitude++)
  {
    if(phase>PI)
    {
   	 b=phase-PI;
     j=PI;
    }
    else
    {
   	 b=0.0;
   	 j=phase;
    }
 	// compute all the line	
   while(b<=j)
 	 {
    xa = (cos(a)*cos(b));
 		ya = (cos(a)*sin(b));
		za = sin(a);
 		xa *= r;
 		ya *= r;
 		za *= r;
 		tab[longitude][0] = (unsigned short)Int_Float(xa*sin(angleX)+ya*-cos(angleX));
 		tab[longitude][1] = (unsigned short)Int_Float(za+ya*sin(angleY));
		b+=SPHERE_PRECISION;
	  longitude++;
   }
	  if( buffer != NULL )
	  {
     sprintf(buffer,"%0.0f%%",((float)longitude/((float)(vertex-1)/100.0)));
     DrawStr((LCD_WIDTH>>1),(LCD_HEIGHT>>1)-3,buffer,A_REPLACE);
	  }
  }
 // erase percentage information on the screen
  if( buffer != NULL )
    DrawStr((LCD_WIDTH>>1),(LCD_HEIGHT>>1)-3,buffer,A_XOR);  

 delta = (short)ceil((float)(longitude/latitude));
 // drawing latitude lines with computed points 
 for(longitude=0,latitude=1;longitude<(vertex-delta);longitude++,latitude++)
 {
  if(latitude>=delta)
  {
   latitude=1;
   longitude++;
  }
  DrawLine(tab[longitude][0]+x,tab[longitude][1]+y,tab[longitude+1][0]+x,tab[longitude+1][1]+y,A_NORMAL);
 } 
 // drawing longitude lines with latitude's points computed before
 for(longitude=0;longitude<(vertex-delta);longitude++)
	 DrawLine(tab[longitude][0]+x,tab[longitude][1]+y,tab[longitude+delta][0]+x,tab[longitude+delta][1]+y,A_NORMAL);
 // restoring memory
 free(buffer);
}

/* ====================================================== */
/* = Function    : Draw the Sky                         = */
/* = Input       : -                                    = */
/* = Output      : -                                    = */                                                                                                                                                                                                                                                                                                                                       
/* ====================================================== */
void DrawSky( void )
{
 short SCR_X = LCD_WIDTH,
 			 SCR_Y = LCD_HEIGHT,
 			 SCRX2 = SCR_X>>1,
 			 SCRY2 = SCR_Y>>1;

 // draw a 360 deg. circular view
 DrawStr(SCRX2,(SCR_Y-7),"S",A_XOR); 	// South
 DrawStr(SCRX2,2,"N",A_XOR); 							// North
 DrawStr(SCRX2-(SCRY2),SCRY2,"E",A_XOR);								// East
 DrawStr(SCRX2+(SCRY2),SCRY2,"W",A_XOR);				// West
 DrawCircle(SCRX2, SCRY2, SCR_Y-20);
 
 ngetchx();
}