/**********************************************************/
/*                        T I A R E                       */
/* Astronomical Ressources & Ephemeris for TI Calculators */
/*               (c) 2003-2005, Yann DUCHEMIN             */
/* ------------------------------------------------------ */
/* This file is part of TIARE.                            */
/*                                                        */
/* TIARE is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public    */
/* License as published by the Free Software Foundation;  */
/* either version 2 of the License                        */
/*                                                        */
/* TIARE is distributed in the hope that it will be       */
/* useful, but WITHOUT ANY WARRANTY; without even the     */
/* implied warranty of MERCHANTABILITY or FITNESS FOR A   */
/* PARTICULAR PURPOSE. See the GNU General Public         */
/* License for more details.                              */
/*                                                        */
/* You should have received a copy of the GNU General     */
/* Public License along with TIARE; if not, write to      */
/* the Free Software Foundation, Inc., 59 Temple Place,   */
/* Suite 330, Boston, MA  02111-1307  USA                 */
/* ------------------------------------------------------ */
/* File Name : saturne.c                                  */
/* Created   : 18 V   2003                                */
/* Modified  : 29 VII 2005                                */
/**********************************************************/

#include "saturne.h"

/* ====================================================== */
/* = Function    : Rings Saturn                         = */
/* = Input       : TIME & Observer structures           = */
/* = Output      : state                                = */
/* ====================================================== */
short RingSat(struct TIME *ptime, struct PLACE *pobs, struct PARAM *pparam)
{
 char *buffer = NULL; //output buffer
 short SCR_X = LCD_WIDTH,
 			 SCR_Y = LCD_HEIGHT,
 			 SCRX2 = SCR_X>>1,
 			 SCRY2 = SCR_Y>>1;
 			 
 float ri = 0,
       ro = 0;
       
 float tva = 0.0,
 			 tvc = 0.0,
 			 dot = 0.0;

 float LocalST = 0.0;
 float jd = 0.0;
 float appd = 0.0;

 struct OBJECT Planet;
 struct OBJECT *pplanet;
 pplanet = &Planet;
 
 // allocate memory for the buffer
 if ((buffer = (char *)calloc(60,sizeof(char))) == NULL)
	return(MSG_MEM);

 // calculate the Julian Day
 jd = YMD2JD(ptime, REALTIME);	// REALTIME or MIDNIGHT
 
 LocalST = RSTG_H(STG_H(YMD2JD(ptime, REALTIME)), DMS2Float(pobs->LongDeg, pobs->LongMin, pobs->LongSec), ptime);

 PosObj(LocalST, ptime, pobs, pplanet, SATURN);

 appd = SDIAMSEC/pplanet->DE;

 DrawStr(SCRX2-3,3,"S",A_NORMAL);
 FontSetSys(F_4x6);
 sprintf(buffer,"\x91: %.1f\"",appd);						// apparent diameter
 DrawStr((SCR_X-40),(SCR_Y-17),buffer, A_XOR);
 sprintf(buffer,"Dist.: %.3f AU",pplanet->DE);		// Earth distance 
 DrawStr((SCR_X-50),(SCR_Y-5),buffer, A_XOR);
 sprintf(buffer,"1\"=%0.1fkm",((Deg2Rad(appd)/3600.0)*(_1AU*pplanet->DE))/(appd));	// evaluated scale in kilometers - a simplified method
 DrawStr((SCR_X-44),(SCR_Y-11),buffer, A_XOR);
 sprintf(buffer,"%02d/%02d/%d %02d:%02d:%02d UTC",ptime->UTYear,ptime->UTMonth,ptime->UTDay,ptime->UTHour,ptime->UTMin,ptime->UTSec);
 DrawStr((SCR_X-90),2,buffer, A_XOR);
 FontSetSys(F_6x8);

 free(buffer);
 
tva = (sinD(pplanet->LAT) * cosD(28.06));
tvc = (cosD(pplanet->LAT) * sinD(28.06));
dot = Rad2Deg(asin( Deg2Rad( tva - tvc * sinD(pplanet->LONG - (169.51 + 3.82E-5 * (YMD2JD(ptime, REALTIME)-2451543.5))) ) ));

 printf("%0.3f\n",(dot));

 pplanet->Ph=180.0; // force angle before to find the angle normalisation in this beta version !!!
 
 // Zoom Corrections
 //validate the planet's size for zooming
 if(((float)(appd*RS2i)*(float)pparam->Zoom) < (float)SCRX2)
 {
  appd=(float)(SATURND*pparam->Zoom);  
  // 0.349 is choosen arbitraly for the angle view from face
  // 1.571 (PI/2) is the start point for X projection, need to add the angle to this
  DrawSphere(SCRX2, SCRY2, (unsigned short)Int_Float(appd), Deg2Rad(pplanet->Ph), 0.349, 1.571);
  // view angle div by planet's angle
  dot /= 90.0;
  ri = (appd*RS1i);
  ro = (ri/(dot));
  DrawClipEllipse (SCRX2, SCRY2, (short)Int_Float(ri), (short)Int_Float(ro), &(SCR_RECT){{0, 0, SCR_X, SCR_Y}}, A_NORMAL);
  ri = (appd*RS1o);
  ro = (ri/(dot));
  DrawClipEllipse (SCRX2, SCRY2, (short)Int_Float(ri), (short)Int_Float(ro), &(SCR_RECT){{0, 0, SCR_X, SCR_Y}}, A_NORMAL);
  ri = (appd*RS2i);
  ro = (ri/(dot));
  DrawClipEllipse (SCRX2, SCRY2, (short)Int_Float(ri), (short)Int_Float(ro), &(SCR_RECT){{0, 0, SCR_X, SCR_Y}}, A_NORMAL);
  ri = (appd*RS2o);
  ro = (ri/(dot));
  DrawClipEllipse (SCRX2, SCRY2, (short)Int_Float(ri), (short)Int_Float(ro), &(SCR_RECT){{0, 0, SCR_X, SCR_Y}}, A_NORMAL);
 }

 ngetchx();
 return(0);
}