/**********************************************************/
/*                        T I A R E                       */
/* Astronomical Ressources & Ephemeris for TI Calculators */
/*                 (c) 2003, Yann DUCHEMIN                */
/* ------------------------------------------------------ */
/* This file is part of TIARE.                            */
/*                                                        */
/* TIARE is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public    */
/* License as published by the Free Software Foundation;  */
/* either version 2 of the License                        */
/*                                                        */
/* TIARE is distributed in the hope that it will be       */
/* useful, but WITHOUT ANY WARRANTY; without even the     */
/* implied warranty of MERCHANTABILITY or FITNESS FOR A   */
/* PARTICULAR PURPOSE. See the GNU General Public         */
/* License for more details.                              */
/*                                                        */
/* You should have received a copy of the GNU General     */
/* Public License along with TIARE; if not, write to      */
/* the Free Software Foundation, Inc., 59 Temple Place,   */
/* Suite 330, Boston, MA  02111-1307  USA                 */
/* ------------------------------------------------------ */
/* File Name : tiare.h                                    */
/* Created   : 27 II  2003                                */
/* Modified  : 10 III 2003                                */
/**********************************************************/

#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200		          // Compile for V200

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define MIN_AMS 100           // Compile for AMS 1.01 or higher

#define SAVE_SCREEN           // Save/Restore LCD Contents

/* ====================================================== */
/* = Poject Definitions                                 = */
/* ====================================================== */
#include <compat.h>
#include <graph.h>
#include <stdio.h>
#include <stdlib.h>
#include <dialogs.h>
#include <args.h>
#include <string.h>
#include <kbd.h>
#include <math.h>
#include <alloc.h>

/* ====================================================== */
/* = Common Definitions                                 = */
/* ====================================================== */
#define		VERSION		"0.01"
#define		CONTACT		"wolfgane@astrosurf.com"

#define		REALTIME	0
#define		MIDNIGHT	1

#define	MSG_INFO	0
#define	MSG_ERR		1
#define	MSG_FILE	2
#define	MSG_NIMP	3
#define	MSG_MEM		4
#define	MSG_END		5
#define MSG_CNT		6
#define MSG_KEY		7
#define MSG_KEY0	8
#define MSG_OBS		9
#define MSG_SEC		10
#define MSG_MIN		11
#define MSG_HRS		12
#define MSG_DAY		13
#define MSG_MTH		14
#define MSG_YER		15
#define END_PRG		99

//theses are reserved for later
#define MAX_OBJ		9	// Max. number of objects' user can define
#define MAX_OBS		3	// Max. number of observer's sites

// create a structure
// to define the observers' coords
// this one doesn't keep any memory because no
// var name is uses
struct PLACE
{
	 short int LatDeg;						// Latitude Degrees
	 unsigned short int LatMin;		// Latitude Minutes
	 unsigned short int LatSec;		// Latitude Secondes
	 short int LongDeg;						// Longitude Degree
	 unsigned short int LongMin;	// Longitude Minutes
	 unsigned short int LongSec;	// Longitude Secondes
	 short int Alt;
	 short int DTime;		// Est(+) & West(-) [TimeZone]
};

// create a struture to save the date & time
// this one doesn't keep any memory because no
// var name is uses
struct TIME
{
	unsigned short int ObsDay;		// Observation Day
	unsigned short int ObsMonth;	//	Month
	unsigned short int ObsYear;						//	Year
	unsigned short int ObsHour;		//  Hour
	unsigned short int ObsMin;		//  Min.
	unsigned short int ObsSec;		//  Sec.
};

// create a structure to save the object relative data
// this one doesn't keep any memory because no
// var name is uses
struct OBJECT
{
	unsigned short LDeg;
	short int LMin;
	short int LSec;
	short int lHour;
	short int lMin;
	short int lSec;
};

// create a structure to convert format DMS
// this one doesn't keep any memory because no
// var name is uses
struct DMS
{
	float d;
	float m;
	float s;
};

// type for the days of the week
// enum DayWeek {Sun, Mon, Tue, Wed, Thu, Fri, Sat};
// type for the month
// enum Month {I, II, III, IV, V, VI, VII, VIII, IX, X, XI, XII};

/* ====================================================== */
/* = tiare.c PROTOTYPES                                 = */
/* ====================================================== */
void _main(void);
inline float Deg2Rad(float);
inline float Rad2Deg(float);
float Int_Float(float);
float NormExp(float);
float NormDeg(float);

/* ====================================================== */
/* = pict.c PROTOTYPES                                  = */
/* ====================================================== */
void DrawScrBorder(void);
void DrawCopyright(void);
void DrawCircle(int, int, int);
short DrawStar(int, int, short);

/* ====================================================== */
/* = moon.c PROTOTYPES                                  = */
/* ====================================================== */
short MoonEffects(struct TIME *, struct PLACE *);

/* ====================================================== */
/* = sun.c PROTOTYPES                                   = */
/* ====================================================== */
short SunEffects(struct TIME *, struct PLACE *);

/* ====================================================== */
/* = menu.c definitions                                 = */
/* ====================================================== */
short MainMenu(struct TIME *, struct PLACE *);
short EdTime(struct TIME *);
short EdPlace(struct PLACE *);

/* ====================================================== */
/* = chronos.c PROTOTYPES                               = */
/* ====================================================== */
short NbDayFMonth(short, short);
short VerifyYearB(short);
float YMD2JD(struct TIME *, short);
float Day2Float(struct TIME *);
float DMS2Float(short , unsigned short , unsigned short);
void Float2DMS(float, struct DMS *);
inline float DayOfWeek(float);
float LST(float ,float, float);
float STG_H(float);
float RSTG_H(float JDM, struct TIME *);
short DST(struct TIME *);
void UTC(struct TIME *, short);

/* ====================================================== */
/* = fich.c PROTOTYPES                                  = */
/* ====================================================== */
short RFchTime(struct TIME *);
short RFchLocal(struct PLACE *);
short FchTime(struct TIME *);
short FchLocal(struct PLACE *);

/* ====================================================== */
/* = errors.c PROTOTYPES                                = */
/* ====================================================== */
void ErrMsg(int, int);
short VerifyTimeStruct(struct TIME *);
short VerifyPlaceStruct(struct PLACE *);

/* ====================================================== */
/* = calend.c PROTOTYPES                                = */
/* ====================================================== */
short Calend(struct TIME *, struct PLACE *);

/* ====================================================== */
/* = jupiter.c PROTOTYPES                               = */
/* ====================================================== */
short JSat(struct TIME *, struct PLACE *);
void DrawGRS(float);

